#include <frost_utils.h>
#include <frost_utilsmanager.h>
#include <frost_utils_timemanager.h>
#include <frost_utils_file.h>

#include <ctime>
#include <iostream>

#include "state.h"
#include "vector.h"

using Frost::s_str; using Frost::s_uint; using Frost::s_double;
using Frost::s_int; using Frost::s_bool; using Frost::s_refptr;
using Frost::s_map; using Frost::s_ctnr; using Frost::s_char;
using Frost::s_wptr; using Frost::nullptr;
using Frost::File; using Frost::UtilsManager; using Frost::TimeManager;
using Frost::Log;

File mLog("tensor.log", File::O);

void LogCout(const s_str& sMessage, const s_bool& bStamps, const s_uint& uiOffset)
{
    std::cout << sMessage.Get() << std::endl;
    mLog.WriteLine(sMessage.Get());
    mLog.Flush();
}

s_double GetTime()
{
    return clock() / (double)CLOCKS_PER_SEC;
}

int main(int argc, char* argv[])
{
    s_ctnr<s_str> lArgs;
    for (int i = 0; i < argc; ++i)
        lArgs.PushBack(argv[i]);

    UtilsManager::GetSingleton()->SetLogFunction(&LogCout);
    TimeManager::GetSingleton()->SetGetTimeFunction(&GetTime);

    State s;

    Vector::Register(s);

    if (lArgs.GetSize() > 1 && File::Exists(lArgs[1]))
    {
        File mTest(lArgs[1]);

        s_str sLine;
        while (mTest.IsValid())
        {
            sLine = mTest.GetLine();

            if (sLine.IsEmpty())
                continue;

            s.ParseLine(sLine);
        }
    }
    else
    {
        // Interractive mode
        s_str sLine;
        while (true)
        {
            std::cout << "> " << std::flush;
            getline(std::cin, sLine.Get());

            if (sLine.IsEmpty())
                continue;

            if (sLine == "q" || sLine == "exit")
                return 0;

            s.ParseLine(sLine);
        }
    }

    TimeManager::Delete();
    UtilsManager::Delete();

    return 0;
}
